@@ -15,14 +15,16 @@
 
 import operator
 
+from neutron_lib.plugins import directory
+
 from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
+from neutron.db import agents_db
 from neutron.db import common_db_mixin as base_db
+from neutron.db import l3_agentschedulers_db as l3_sch_db
 from neutron.extensions import l3
-from neutron import manager
-from neutron.plugins.common import constants as p_const
-from neutron_lib import constants
+from neutron_lib import constants as nl_constants
 from neutron_lib.db import model_base
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -34,6 +36,9 @@ from sqlalchemy.orm import exc
 
 import netaddr
 
+from neutron_fwaas.common import fwaas_constants
+from neutron_fwaas.db.firewall import firewall_router_insertion_db \
+    as fw_r_ins_db
 from neutron_fwaas.extensions import firewall as fw_ext
 
 
@@ -100,7 +105,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
 
     @property
     def _core_plugin(self):
-        return manager.NeutronManager.get_plugin()
+        return directory.get_plugin()
 
     def _get_firewall(self, context, id):
         try:
@@ -295,12 +300,12 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
 
     def _validate_fwr_src_dst_ip_version(self, fwr):
         src_version = dst_version = None
-        if fwr['source_ip_address']:
+        if fwr.get('source_ip_address', None):
             src_version = netaddr.IPNetwork(fwr['source_ip_address']).version
-        if fwr['destination_ip_address']:
+        if fwr.get('destination_ip_address', None):
             dst_version = netaddr.IPNetwork(
                 fwr['destination_ip_address']).version
-        rule_ip_version = fwr['ip_version']
+        rule_ip_version = fwr.get('ip_version', None)
         if ((src_version and src_version != rule_ip_version) or
                 (dst_version and dst_version != rule_ip_version)):
             raise fw_ext.FirewallIpAddressConflict()
@@ -311,10 +316,11 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             raise fw_ext.FirewallRuleInvalidPortValue(port=port_range)
 
     def _validate_fwr_protocol_parameters(self, fwr):
-        protocol = fwr['protocol']
-        if protocol not in (constants.PROTO_NAME_TCP,
-                            constants.PROTO_NAME_UDP):
-            if fwr['source_port'] or fwr['destination_port']:
+        protocol = fwr.get('protocol', None)
+        if protocol not in (nl_constants.PROTO_NAME_TCP,
+                            nl_constants.PROTO_NAME_UDP):
+            if (fwr.get('source_port', None) or
+                    fwr.get('destination_port', None)):
                 raise fw_ext.FirewallRuleInvalidICMPParameter(
                     param="Source, destination port")
 
@@ -326,8 +332,8 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         # the introduction of a new 'CREATED' state allows this, whilst
         # keeping a backward compatible behavior of the logical resource.
         if not status:
-            status = (p_const.CREATED if cfg.CONF.router_distributed
-                      else p_const.PENDING_CREATE)
+            status = (nl_constants.CREATED if cfg.CONF.router_distributed
+                      else nl_constants.PENDING_CREATE)
         with context.session.begin(subtransactions=True):
             self._validate_fw_parameters(context, fw, tenant_id)
             firewall_db = Firewall(
@@ -490,6 +496,8 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
     def update_firewall_rule(self, context, id, firewall_rule):
         LOG.debug("update_firewall_rule() called")
         fwr = firewall_rule['firewall_rule']
+        self._validate_fwr_protocol_parameters(fwr)
+        self._validate_fwr_src_dst_ip_version(fwr)
         fwr_db = self._get_firewall_rule(context, id)
         if fwr_db.firewall_policy_id:
             fwp_db = self._get_firewall_policy(context,
@@ -611,12 +619,22 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                     firewall_policy_id=id)
             return self._process_rule_for_policy(context, id, fwr_db, None)
 
+    def get_firewall_tenant_ids_on_host(self, context, host):
+        query = context.session.query(Firewall.tenant_id)
+        query = query.join(fw_r_ins_db.FirewallRouterAssociation)
+        query = query.join(l3_sch_db.RouterL3AgentBinding,
+                           l3_sch_db.RouterL3AgentBinding.router_id ==
+                           fw_r_ins_db.FirewallRouterAssociation.router_id)
+        query = query.join(agents_db.Agent)
+        query = query.filter(agents_db.Agent.host == host)
+        query = query.distinct()
+        return [item[0] for item in query]
+
 
 def migration_callback(resource, event, trigger, **kwargs):
     context = kwargs['context']
     router = kwargs['router']
-    fw_plugin = manager.NeutronManager.get_service_plugins().get(
-        p_const.FIREWALL)
+    fw_plugin = directory.get_plugin(fwaas_constants.FIREWALL)
     if fw_plugin:
         tenant_firewalls = fw_plugin.get_firewalls(
             context, filters={'tenant_id': [router['tenant_id']]})
