@@ -13,12 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron_lib import constants
+from neutron_lib import constants as nl_constants
 from neutron_lib.plugins import directory
 
 from neutron.common import rpc as n_rpc
+from neutron.common import utils as n_utils
 from neutron import context as neutron_context
-from neutron_lib import constants as nl_constants
+
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
@@ -108,10 +109,10 @@ class FirewallCallbacks(object):
     def get_tenants_with_firewalls(self, context, **kwargs):
         """Agent uses this to get all tenants that have firewalls."""
         LOG.debug("get_tenants_with_firewalls() called")
+        host = kwargs['host']
         ctx = neutron_context.get_admin_context()
-        fw_list = self.plugin.get_firewalls(ctx)
-        fw_tenant_list = list(set(fw['tenant_id'] for fw in fw_list))
-        return fw_tenant_list
+        tenant_ids = self.plugin.get_firewall_tenant_ids_on_host(ctx, host)
+        return tenant_ids
 
 
 class FirewallAgentApi(object):
@@ -122,18 +123,28 @@ class FirewallAgentApi(object):
         target = oslo_messaging.Target(topic=topic, version='1.0')
         self.client = n_rpc.get_client(target)
 
-    def create_firewall(self, context, firewall):
-        cctxt = self.client.prepare(fanout=True)
+    def _prepare_rpc_client(self, host=None):
+        if host:
+            return self.client.prepare(server=host)
+        else:
+            # historical behaviour (RPC broadcast)
+            return self.client.prepare(fanout=True)
+
+    def create_firewall(self, context, firewall, host=None):
+        cctxt = self._prepare_rpc_client(host)
+        # TODO(blallau) host param is not used on agent side (to be removed)
         cctxt.cast(context, 'create_firewall', firewall=firewall,
                    host=self.host)
 
-    def update_firewall(self, context, firewall):
-        cctxt = self.client.prepare(fanout=True)
+    def update_firewall(self, context, firewall, host=None):
+        cctxt = self._prepare_rpc_client(host)
+        # TODO(blallau) host param is not used on agent side (to be removed)
         cctxt.cast(context, 'update_firewall', firewall=firewall,
                    host=self.host)
 
-    def delete_firewall(self, context, firewall):
-        cctxt = self.client.prepare(fanout=True)
+    def delete_firewall(self, context, firewall, host=None):
+        cctxt = self._prepare_rpc_client(host)
+        # TODO(blallau) host param is not used on agent side (to be removed)
         cctxt.cast(context, 'delete_firewall', firewall=firewall,
                    host=self.host)
 
@@ -169,6 +180,21 @@ class FirewallPlugin(
             f_const.FIREWALL_PLUGIN, self.endpoints, fanout=False)
         return self.conn.consume_in_threads()
 
+    def _get_hosts_to_notify(self, context, router_ids):
+        """Returns all hosts to send notification about firewall update"""
+        l3_plugin = directory.get_plugin(nl_constants.L3)
+        no_broadcast = (
+            n_utils.is_extension_supported(
+                l3_plugin, nl_constants.L3_AGENT_SCHEDULER_EXT_ALIAS) and
+            getattr(l3_plugin, 'get_l3_agents_hosting_routers', False))
+        if no_broadcast:
+            agents = l3_plugin.get_l3_agents_hosting_routers(
+                context, router_ids, admin_state_up=True, active=True)
+            return [a.host for a in agents]
+
+        # NOTE(blallau): default: FirewallAgentAPI performs RPC broadcast
+        return [None]
+
     def _rpc_update_firewall(self, context, firewall_id):
         status_update = {"firewall": {"status": nl_constants.PENDING_UPDATE}}
         super(FirewallPlugin, self).update_firewall(context, firewall_id,
@@ -177,10 +203,14 @@ class FirewallPlugin(
                                                             firewall_id)
         # this is triggered on an update to fw rule or policy, no
         # change in associated routers.
-        fw_with_rules['add-router-ids'] = self.get_firewall_routers(
-                context, firewall_id)
+        fw_update_rtrs = self.get_firewall_routers(context, firewall_id)
+        fw_with_rules['add-router-ids'] = fw_update_rtrs
         fw_with_rules['del-router-ids'] = []
-        self.agent_rpc.update_firewall(context, fw_with_rules)
+
+        hosts = self._get_hosts_to_notify(context, fw_update_rtrs)
+        for host in hosts:
+            self.agent_rpc.update_firewall(context, fw_with_rules,
+                                           host=host)
 
     def _rpc_update_firewall_policy(self, context, firewall_policy_id):
         firewall_policy = self.get_firewall_policy(context, firewall_policy_id)
@@ -215,7 +245,7 @@ class FirewallPlugin(
         if router_ids == nl_constants.ATTR_NOT_SPECIFIED:
             # old semantics router-ids keyword not specified pick up
             # all routers on tenant.
-            l3_plugin = directory.get_plugin(constants.L3)
+            l3_plugin = directory.get_plugin(nl_constants.L3)
             ctx = neutron_context.get_admin_context()
             routers = l3_plugin.get_routers(ctx)
             router_ids = [
@@ -263,8 +293,10 @@ class FirewallPlugin(
         fw_with_rules['add-router-ids'] = fw_new_rtrs
         fw_with_rules['del-router-ids'] = []
 
-        self.agent_rpc.create_firewall(context, fw_with_rules)
-
+        hosts = self._get_hosts_to_notify(context, fw_new_rtrs)
+        for host in hosts:
+            self.agent_rpc.create_firewall(context, fw_with_rules,
+                                           host=host)
         return fw
 
     def update_firewall(self, context, id, firewall):
@@ -320,8 +352,11 @@ class FirewallPlugin(
             fw_with_rules['add-router-ids'],
             fw_with_rules['del-router-ids'])
 
-        self.agent_rpc.update_firewall(context, fw_with_rules)
-
+        hosts = self._get_hosts_to_notify(context, list(
+            set(fw_new_rtrs).union(set(fw_current_rtrs))))
+        for host in hosts:
+            self.agent_rpc.update_firewall(context, fw_with_rules,
+                                           host=host)
         return fw
 
     def delete_db_firewall_object(self, context, id):
@@ -331,8 +366,8 @@ class FirewallPlugin(
         LOG.debug("delete_firewall() called on firewall %s", id)
         fw_with_rules = (
             self._make_firewall_dict_with_rules(context, id))
-        fw_with_rules['del-router-ids'] = self.get_firewall_routers(
-            context, id)
+        fw_delete_rtrs = self.get_firewall_routers(context, id)
+        fw_with_rules['del-router-ids'] = fw_delete_rtrs
         fw_with_rules['add-router-ids'] = []
         if not fw_with_rules['del-router-ids']:
             # no routers to delete on the agent side
@@ -342,7 +377,15 @@ class FirewallPlugin(
             super(FirewallPlugin, self).update_firewall(context, id, status)
             # Reflect state change in fw_with_rules
             fw_with_rules['status'] = status['firewall']['status']
-            self.agent_rpc.delete_firewall(context, fw_with_rules)
+            hosts = self._get_hosts_to_notify(context, fw_delete_rtrs)
+            if hosts:
+                for host in hosts:
+                    self.agent_rpc.delete_firewall(context, fw_with_rules,
+                                                   host=host)
+            else:
+                # NOTE(blallau): we directly delete the firewall
+                # if router is not associated to an agent
+                self.delete_db_firewall_object(context, id)
 
     def update_firewall_policy(self, context, id, firewall_policy):
         LOG.debug("update_firewall_policy() called")
