@@ -16,14 +16,12 @@
 import time
 
 from tempest import config
-from tempest import exceptions
 from tempest.lib.common.utils import data_utils
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions as lib_exc
 
-from neutron.plugins.common import constants as p_const
-
 from neutron_fwaas.tests.tempest_plugin.services import client
+from neutron_lib import constants as nl_constants
 
 CONF = config.CONF
 
@@ -91,13 +89,13 @@ class FWaaSClientMixin(object):
 
     def delete_firewall_and_wait(self, firewall_id):
         self.firewalls_client.delete_firewall(firewall_id)
-        self._wait_firewall_while(firewall_id, [p_const.PENDING_DELETE],
+        self._wait_firewall_while(firewall_id, [nl_constants.PENDING_DELETE],
                                   not_found_ok=True)
 
     def _wait_firewall_ready(self, firewall_id):
         self._wait_firewall_while(firewall_id,
-                                  [p_const.PENDING_CREATE,
-                                   p_const.PENDING_UPDATE])
+                                  [nl_constants.PENDING_CREATE,
+                                   nl_constants.PENDING_UPDATE])
 
     def _wait_firewall_while(self, firewall_id, statuses, not_found_ok=False):
         start = int(time.time())
@@ -119,5 +117,5 @@ class FWaaSClientMixin(object):
                     "firewall": firewall_id,
                     "status": status,
                 }
-                raise exceptions.TimeoutException(msg)
+                raise lib_exc.TimeoutException(msg)
             time.sleep(1)
