@@ -19,12 +19,10 @@ import mock
 from neutron.api import extensions as api_ext
 from neutron.common import config
 from neutron import context
-import neutron.extensions as nextensions
-from neutron import manager
-from neutron.plugins.common import constants
 from oslo_config import cfg
 from oslo_utils import importutils
 from oslo_utils import uuidutils
+
 import six
 import testtools
 import webob.exc
@@ -34,13 +32,15 @@ from neutron_fwaas import extensions
 from neutron_fwaas.extensions import firewall_v2 as firewall
 from neutron_fwaas.services.firewall import fwaas_plugin_v2
 from neutron_fwaas.tests import base
+from neutron_lib import constants as nl_constants
+from neutron_lib.plugins import directory
 
 DB_FW_PLUGIN_KLASS = (
     "neutron_fwaas.db.firewall.v2.firewall_db_v2.Firewall_db_mixin_v2"
 )
 FWAAS_PLUGIN = 'neutron_fwaas.services.firewall.fwaas_plugin_v2'
 DELETEFW_PATH = FWAAS_PLUGIN + '.FirewallAgentApi.delete_firewall_group'
-extensions_path = ':'.join(extensions.__path__ + nextensions.__path__)
+extensions_path = ':'.join(extensions.__path__)
 DESCRIPTION = 'default description'
 PUBLIC = True
 PROTOCOL = 'tcp'
@@ -68,8 +68,7 @@ class FakeAgentApi(fwaas_plugin_v2.FirewallCallbacks):
         pass
 
     def delete_firewall_group(self, context, firewall_group, **kwargs):
-        self.plugin = (manager.NeutronManager.
-            get_service_plugins()['FIREWALL_V2'])
+        self.plugin = directory.get_plugin('FIREWALL_V2')
         self.firewall_group_deleted(context, firewall_group['id'], **kwargs)
 
 
@@ -99,10 +98,18 @@ class FirewallPluginV2DbTestCase(base.NeutronDbPluginV2TestCase):
             self.plugin = importutils.import_object(fw_plugin)
             ext_mgr = api_ext.PluginAwareExtensionManager(
                 extensions_path,
-                {constants.FIREWALL: self.plugin}
+                {'FIREWALL': self.plugin}
             )
             app = config.load_paste_app('extensions_test_app')
             self.ext_api = api_ext.ExtensionMiddleware(app, ext_mgr=ext_mgr)
+        router_distributed_opts = [
+            cfg.BoolOpt(
+                'router_distributed',
+                default=False,
+                help=_("System-wide flag to determine the type of router "
+                       "that tenants can create. Only admin can override.")),
+        ]
+        cfg.CONF.register_opts(router_distributed_opts)
 
     def _test_list_resources(self, resource, items,
                              neutron_context=None,
@@ -856,6 +863,7 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
         attrs['destination_port'] = '30:40'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': '10:20',
                                       'destination_port': '30:40'}}
             req = self.new_update_request('firewall_rules', data,
@@ -869,6 +877,7 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
         attrs['destination_port'] = '80'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': 10000,
                                       'destination_port': 80}}
             req = self.new_update_request('firewall_rules', data,
@@ -882,6 +891,7 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
         attrs['destination_port'] = '80'
         with self.firewall_rule() as fwr:
             data = {'firewall_rule': {'name': name,
+                                      'protocol': PROTOCOL,
                                       'source_port': '10000',
                                       'destination_port': '80'}}
             req = self.new_update_request('firewall_rules', data,
@@ -952,6 +962,26 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
             res = req.get_response(self.ext_api)
             self.assertEqual(200, res.status_int)
 
+    def test_update_firewall_rule_icmp_with_port(self):
+        with self.firewall_rule(source_port=None,
+                                destination_port=None,
+                                protocol=None) as fwr:
+            data = {'firewall_rule': {'destination_port': 80,
+                                      'protocol': 'icmp'}}
+            req = self.new_update_request('firewall_rules', data,
+                                          fwr['firewall_rule']['id'])
+            res = req.get_response(self.ext_api)
+            self.assertEqual(400, res.status_int)
+
+        with self.firewall_rule(source_port=None,
+                                destination_port=None,
+                                protocol='icmp') as fwr:
+            data = {'firewall_rule': {'destination_port': 80}}
+            req = self.new_update_request('firewall_rules', data,
+                                          fwr['firewall_rule']['id'])
+            res = req.get_response(self.ext_api)
+            self.assertEqual(400, res.status_int)
+
     def test_update_firewall_rule_with_policy_associated(self):
         name = "new_firewall_rule1"
         attrs = self._get_test_firewall_rule_attrs(name)
@@ -991,6 +1021,17 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                 res = req.get_response(self.ext_api)
                 self.assertEqual(webob.exc.HTTPConflict.code, res.status_int)
 
+    def test_update_firewall_rule_with_ipv6_ipaddr(self):
+        with self.firewall_rule(source_ip_address="1::10",
+                                destination_ip_address=None,
+                                ip_version=6) as fwr_v6:
+            data = {'firewall_rule': {
+                'destination_ip_address': "2::20"}}
+            req = self.new_update_request('firewall_rules', data,
+                                          fwr_v6['firewall_rule']['id'])
+            res = req.get_response(self.ext_api)
+            self.assertEqual(200, res.status_int)
+
     def test_delete_firewall_rule(self):
         ctx = context.get_admin_context()
         with self.firewall_rule(do_delete=False) as fwr:
@@ -1023,7 +1064,8 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                 name=attrs['name'],
                 ingress_firewall_policy_id=fwp_id,
                 egress_firewall_policy_id=fwp_id,
-                admin_state_up=ADMIN_STATE_UP
+                admin_state_up=ADMIN_STATE_UP,
+                ports=attrs['ports'] if 'ports' in attrs else None,
             ) as firewall_group:
                 for k, v in six.iteritems(attrs):
                     self.assertEqual(v, firewall_group['firewall_group'][k])
@@ -1032,6 +1074,18 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
         attrs = self._get_test_firewall_group_attrs("firewall1")
         self._test_create_firewall_group(attrs)
 
+    def test_create_firewall_group_with_ports(self):
+        with self.port(
+            device_owner=nl_constants.DEVICE_OWNER_ROUTER_INTF) as dummy_port:
+            attrs = self._get_test_firewall_group_attrs("fwg1")
+            attrs['ports'] = [dummy_port['port']['id']]
+            self._test_create_firewall_group(attrs)
+
+    def test_create_firewall_group_with_empty_ports(self):
+        attrs = self._get_test_firewall_group_attrs("fwg1")
+        attrs['ports'] = []
+        self._test_create_firewall_group(attrs)
+
     def test_create_firewall_group_with_dvr(self):
         cfg.CONF.set_override('router_distributed', True)
         attrs = self._get_test_firewall_group_attrs("firewall1", "CREATED")
@@ -1088,17 +1142,15 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                 self.assertEqual(target_tenant,
                                  fwg['firewall_group']['tenant_id'])
 
-    def test_show_firewall_group(self):
-        name = "firewall1"
-        attrs = self._get_test_firewall_group_attrs(name)
-
+    def _test_show_firewall_group(self, attrs):
         with self.firewall_policy() as fwp:
             fwp_id = fwp['firewall_policy']['id']
             attrs['ingress_firewall_policy_id'] = fwp_id
             attrs['egress_firewall_policy_id'] = fwp_id
             attrs['status'] = 'PENDING_CREATE'
             with self.firewall_group(
-                    name=name,
+                    name=attrs['name'],
+                    ports=attrs['ports'] if 'ports' in attrs else None,
                     ingress_firewall_policy_id=fwp_id,
                     egress_firewall_policy_id=fwp_id,
                     admin_state_up=ADMIN_STATE_UP) as firewall_group:
@@ -1106,11 +1158,27 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                     'firewall_groups',
                     firewall_group['firewall_group']['id'],
                     fmt=self.fmt)
-                res = self.deserialize(self.fmt,
-                                       req.get_response(self.ext_api))
+                res = self.deserialize(
+                    self.fmt, req.get_response(self.ext_api))
                 for k, v in six.iteritems(attrs):
                     self.assertEqual(v, res['firewall_group'][k])
 
+    def test_show_firewall_group(self):
+        attrs = self._get_test_firewall_group_attrs('fwg1')
+        self._test_show_firewall_group(attrs)
+
+    def test_show_firewall_group_with_ports(self):
+        attrs = self._get_test_firewall_group_attrs('fwg1')
+        with self.port(
+            device_owner=nl_constants.DEVICE_OWNER_ROUTER_INTF) as dummy_port:
+            attrs['ports'] = [dummy_port['port']['id']]
+            self._test_show_firewall_group(attrs)
+
+    def test_show_firewall_group_with_empty_ports(self):
+        attrs = self._get_test_firewall_group_attrs('fwg1')
+        attrs['ports'] = []
+        self._test_show_firewall_group(attrs)
+
     def test_list_firewall_groups(self):
         with self.firewall_policy() as fwp:
             fwp_id = fwp['firewall_policy']['id']
@@ -1213,7 +1281,7 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                                context=ctx) as firewall:
                 fw_id = firewall['firewall_group']['id']
                 fw_db = self.plugin._get_firewall_group(ctx, fw_id)
-                fw_db['status'] = constants.ACTIVE
+                fw_db['status'] = nl_constants.ACTIVE
                 # update firewall from fwp1 to fwp2(different tenant)
                 data = {'firewall_group':
                         {'ingress_firewall_policy_id': fwp2_id}}
@@ -1294,7 +1362,7 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                 insert_data = {'firewall_rule_id': None,
                                'insert_before': '123',
                                'insert_after': '456'}
-                self._rule_action('insert', fwp_id, '123',
+                self._rule_action('insert', fwp_id, None,
                                   expected_code=webob.exc.HTTPNotFound.code,
                                   expected_body=None,
                                   body_data=insert_data)
@@ -1307,6 +1375,22 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                                   expected_code=webob.exc.HTTPBadRequest.code,
                                   expected_body=None)
 
+    def test_insert_rule_and_already_associated(self):
+        with self.firewall_rule() as fwr:
+            fwr_id = fwr['firewall_rule']['id']
+            with self.firewall_policy(firewall_rules=[fwr_id]) as fwp:
+                fwp_id = fwp['firewall_policy']['id']
+                msg = "Operation cannot be performed since Firewall Rule " \
+                      "{0} is already associated with FirewallPolicy " \
+                      "{1}".format(fwr_id, fwp_id)
+                result = self._rule_action(
+                    'insert', fwp_id, fwr_id,
+                    insert_before=None,
+                    insert_after=None,
+                    expected_code=webob.exc.HTTPConflict.code,
+                    body_data={'firewall_rule_id': fwr_id})
+                self.assertEqual(msg, result['NeutronError']['message'])
+
     def test_insert_rule_for_previously_associated_rule(self):
         with self.firewall_rule() as fwr:
             fwr_id = fwr['firewall_rule']['id']
@@ -1446,6 +1530,21 @@ class TestFirewallDBPluginV2(FirewallPluginV2DbTestCase):
                                   expected_code=webob.exc.HTTPOk.code,
                                   expected_body=attrs)
 
+    def test_remove_rule_and_not_associated(self):
+        with self.firewall_rule(name='fwr0') as fwr:
+            with self.firewall_policy(name='firewall_policy2') as fwp:
+                fwp_id = fwp['firewall_policy']['id']
+                fwr_id = fwr['firewall_rule']['id']
+                msg = "Firewall Rule {0} is not associated with " \
+                      "Firewall Policy {1}.".format(fwr_id, fwp_id)
+                result = self._rule_action(
+                    'remove', fwp_id, fwr_id,
+                    insert_before=None,
+                    insert_after=None,
+                    expected_code=webob.exc.HTTPBadRequest.code,
+                    body_data={'firewall_rule_id': fwr_id})
+                self.assertEqual(msg, result['NeutronError']['message'])
+
     def test_remove_rule_from_policy(self):
         attrs = self._get_test_firewall_policy_attrs()
         attrs['audited'] = False
