@@ -34,13 +34,15 @@ FW_LEGACY = 'legacy'
 class IptablesFwaasTestCase(base.BaseTestCase):
     def setUp(self):
         super(IptablesFwaasTestCase, self).setUp()
-        self.utils_exec_p = mock.patch(
-            'neutron.agent.linux.utils.execute')
-        self.utils_exec = self.utils_exec_p.start()
+        self.conntrack_driver = mock.Mock()
+        self.conntrack_driver.initialize = mock.Mock()
+        self.conntrack_driver.delete_entries = mock.Mock()
+        self.conntrack_driver.flush_entries = mock.Mock()
         self.iptables_cls_p = mock.patch(
             'neutron.agent.linux.iptables_manager.IptablesManager')
         self.iptables_cls_p.start()
         self.firewall = fwaas.IptablesFwaasDriver()
+        self.firewall.conntrack = self.conntrack_driver
 
     def _fake_rules_v4(self, fwid, apply_list):
         rule_list = []
@@ -285,11 +287,8 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         self.firewall.create_firewall(FW_LEGACY, apply_list, firewall)
         for router_info_inst in apply_list:
             namespace = router_info_inst.iptables_manager.namespace
-            cmd = ['ip', 'netns', 'exec', namespace, 'conntrack', '-D']
-            calls = [
-                mock.call(cmd, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1])]
-            self.utils_exec.assert_has_calls(calls)
+            calls = [mock.call(namespace)]
+            self.conntrack_driver.flush_entries.assert_has_calls(calls)
 
     def test_remove_conntrack_inserted_rule(self):
         apply_list = self._fake_apply_list()
@@ -305,18 +304,35 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         rule_list.insert(2, insert_rule)
         firewall = self._fake_firewall(rule_list)
         self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
+        rules_changed = [
+            {'destination_port': '23',
+             'position': '2',
+             'protocol': 'tcp',
+             'ip_version': 4,
+             'enabled': True,
+             'action': 'reject',
+             'id': 'fake-fw-rule3'},
+            {'destination_port': '23',
+             'position': '3',
+             'protocol': 'tcp',
+             'ip_version': 4,
+             'enabled': True,
+             'action': 'reject',
+             'id': 'fake-fw-rule3'}
+        ]
+        rules_inserted = [
+            {'id': 'fake-fw-rule',
+             'protocol': 'icmp',
+             'ip_version': 4,
+             'enabled': True,
+             'action': 'deny',
+             'position': '2'}
+        ]
         for router_info_inst in apply_list:
             namespace = router_info_inst.iptables_manager.namespace
-            cmd1 = ['ip', 'netns', 'exec', namespace, 'conntrack',
-                    '-D', '-p', 'tcp', '-f', 'ipv4', '--dport', '23']
-            cmd2 = ['ip', 'netns', 'exec', namespace, 'conntrack',
-                    '-D', '-p', 'icmp', '-f', 'ipv4']
-            calls = [
-                mock.call(cmd1, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1]),
-                mock.call(cmd2, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1])]
-            self.utils_exec.assert_has_calls(calls)
+            self.conntrack_driver.delete_entries.assert_called_once_with(
+                rules_changed + rules_inserted, namespace
+            )
 
     def test_remove_conntrack_removed_rule(self):
         apply_list = self._fake_apply_list()
@@ -328,18 +344,36 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         rule_list.remove(remove_rule)
         firewall = self._fake_firewall(rule_list)
         self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
+        rules_changed = [
+            {'destination_port': '23',
+             'position': '2',
+             'protocol': 'tcp',
+             'ip_version': 4,
+             'enabled': True,
+             'action': 'reject',
+             'id': 'fake-fw-rule3'},
+            {'destination_port': '23',
+             'position': '1',
+             'protocol': 'tcp',
+             'ip_version': 4,
+             'enabled': True,
+             'action': 'reject',
+             'id': 'fake-fw-rule3'}
+        ]
+        rules_removed = [
+            {'enabled': True,
+             'position': '1',
+             'protocol': 'tcp',
+             'id': 'fake-fw-rule2',
+             'ip_version': 4,
+             'action': 'deny',
+             'destination_port': '22'}
+        ]
         for router_info_inst in apply_list:
             namespace = router_info_inst.iptables_manager.namespace
-            cmd1 = ['ip', 'netns', 'exec', namespace, 'conntrack',
-                    '-D', '-p', 'tcp', '-f', 'ipv4', '--dport', '23']
-            cmd2 = ['ip', 'netns', 'exec', namespace, 'conntrack',
-                    '-D', '-p', 'tcp', '-f', 'ipv4', '--dport', '22']
-            calls = [
-                mock.call(cmd1, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1]),
-                mock.call(cmd2, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1])]
-            self.utils_exec.assert_has_calls(calls)
+            self.conntrack_driver.delete_entries.assert_called_once_with(
+                rules_changed + rules_removed, namespace
+            )
 
     def test_remove_conntrack_changed_rule(self):
         apply_list = self._fake_apply_list()
@@ -349,20 +383,28 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         income_rule = {'enabled': True,
                  'action': 'deny',
                  'ip_version': 4,
-                 'protocol': 'icmp',
-                 'id': 'fake-fw-rule2'}
-        rule_list[1] = income_rule
+                 'protocol': 'tcp',
+                 'id': 'fake-fw-rule3'}
+        rule_list[2] = income_rule
         firewall = self._fake_firewall(rule_list)
         self.firewall.update_firewall(FW_LEGACY, apply_list, firewall)
+        rules_changed = [
+            {'id': 'fake-fw-rule3',
+             'enabled': True,
+             'action': 'reject',
+             'position': '2',
+             'destination_port': '23',
+             'ip_version': 4,
+             'protocol': 'tcp'},
+            {'position': '2',
+             'enabled': True,
+             'action': 'deny',
+             'id': 'fake-fw-rule3',
+             'ip_version': 4,
+             'protocol': 'tcp'}
+        ]
         for router_info_inst in apply_list:
             namespace = router_info_inst.iptables_manager.namespace
-            cmd1 = ['ip', 'netns', 'exec', namespace, 'conntrack', '-D',
-                    '-p', 'tcp', '-f', 'ipv4', '--dport', '22']
-            cmd2 = ['ip', 'netns', 'exec', namespace, 'conntrack', '-D',
-                    '-p', 'icmp', '-f', 'ipv4']
-            calls = [
-                mock.call(cmd1, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1]),
-                mock.call(cmd2, run_as_root=True, check_exit_code=True,
-                      extra_ok_codes=[1])]
-            self.utils_exec.assert_has_calls(calls)
+            self.conntrack_driver.delete_entries.assert_called_once_with(
+                rules_changed, namespace
+            )
